# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json
from unittest import mock
from unittest.mock import AsyncMock, MagicMock, call, patch

import apprise
import pytest
from apprise import NotifyFormat, NotifyType

from airflow.models import Connection
from airflow.providers.apprise.hooks.apprise import AppriseHook


class TestAppriseHook:
    """
    Test for AppriseHook
    """

    @pytest.mark.parametrize(
        "config",
        [
            {"path": "http://some_path_that_dont_exist/", "tag": "alert"},
            '{"path": "http://some_path_that_dont_exist/", "tag": "alert"}',
        ],
    )
    def test_get_config_from_conn(self, config):
        extra = {"config": config}
        conn = Connection(conn_type="apprise", extra=extra)
        hook = AppriseHook()
        assert hook.get_config_from_conn(conn) == (json.loads(config) if isinstance(config, str) else config)

    def test_set_config_from_conn_with_dict(self):
        """
        Test set_config_from_conn for dict config
        """
        extra = {"config": {"path": "http://some_path_that_dont_exist/", "tag": "alert"}}
        apprise_obj = apprise.Apprise()
        apprise_obj.add = MagicMock()
        conn = Connection(conn_type="apprise", extra=extra)
        hook = AppriseHook()
        hook.set_config_from_conn(conn=conn, apprise_obj=apprise_obj)

        apprise_obj.add.assert_called_once_with("http://some_path_that_dont_exist/", tag="alert")

    def test_set_config_from_conn_with_list(self):
        """
        Test set_config_from_conn for list of dict config
        """
        extra = {
            "config": [
                {"path": "http://some_path_that_dont_exist/", "tag": "p0"},
                {"path": "http://some_other_path_that_dont_exist/", "tag": "p1"},
            ]
        }

        apprise_obj = apprise.Apprise()
        apprise_obj.add = MagicMock()
        conn = Connection(conn_type="apprise", extra=extra)
        hook = AppriseHook()
        hook.set_config_from_conn(conn=conn, apprise_obj=apprise_obj)

        apprise_obj.add.assert_has_calls(
            [
                call("http://some_path_that_dont_exist/", tag="p0"),
                call("http://some_other_path_that_dont_exist/", tag="p1"),
            ]
        )

    @mock.patch(
        "airflow.providers.apprise.hooks.apprise.AppriseHook.get_connection",
    )
    def test_notify(self, mock_conn):
        mock_conn.return_value = Connection(
            conn_id="apprise",
            extra={
                "config": [
                    {"path": "http://some_path_that_dont_exist/", "tag": "p0"},
                    {"path": "http://some_other_path_that_dont_exist/", "tag": "p1"},
                ]
            },
        )
        apprise_obj = apprise.Apprise()
        apprise_obj.notify = MagicMock()
        apprise_obj.add = MagicMock()
        with patch.object(apprise, "Apprise", return_value=apprise_obj):
            hook = AppriseHook()
            hook.notify(body="test")

        apprise_obj.notify.assert_called_once_with(
            body="test",
            title="",
            notify_type=NotifyType.INFO,
            body_format=NotifyFormat.TEXT,
            tag="all",
            attach=None,
            interpret_escapes=None,
        )

    @pytest.mark.asyncio
    @mock.patch(
        "airflow.providers.apprise.hooks.apprise.get_async_connection",
    )
    async def test_async_notify(self, mock_conn):
        mock_conn.return_value = Connection(
            conn_id="apprise",
            extra={
                "config": [
                    {"path": "http://some_path_that_dont_exist/", "tag": "p0"},
                    {"path": "http://some_other_path_that_dont_exist/", "tag": "p1"},
                ]
            },
        )
        apprise_obj = apprise.Apprise()
        apprise_obj.async_notify = AsyncMock()
        apprise_obj.add = MagicMock()
        with patch.object(apprise, "Apprise", return_value=apprise_obj):
            hook = AppriseHook()
            await hook.async_notify(body="test")

        mock_conn.assert_called()
        apprise_obj.async_notify.assert_called_once_with(
            body="test",
            title="",
            notify_type=NotifyType.INFO,
            body_format=NotifyFormat.TEXT,
            tag="all",
            attach=None,
            interpret_escapes=None,
        )



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

from __future__ import annotations

from collections.abc import Iterable
from functools import cached_property

from apprise import AppriseConfig, NotifyFormat, NotifyType

from airflow.providers.apprise.hooks.apprise import AppriseHook
from airflow.providers.apprise.version_compat import AIRFLOW_V_3_1_PLUS
from airflow.providers.common.compat.notifier import BaseNotifier


class AppriseNotifier(BaseNotifier):
    r"""
    Apprise BaseNotifier.

    :param body: Specify the message body
    :param title: Specify the message title. This field is complete optional
    :param notify_type: Specify the message type (default=info). Possible values are "info",
        "success", "failure", and "warning"
    :param body_format: Specify the input message format (default=text). Possible values are "text",
        "html", and "markdown".
    :param tag: Specify one or more tags to filter which services to notify
    :param attach: Specify one or more file attachment locations
    :param interpret_escapes: Enable interpretation of backslash escapes. For example, this would convert
        sequences such as \n and \r to their respected ascii new-line and carriage
    :param config: Specify one or more configuration
    :param apprise_conn_id: connection that has Apprise configs setup
    """

    template_fields = ("body", "title", "tag", "attach")

    def __init__(
        self,
        *,
        body: str,
        title: str | None = None,
        notify_type: NotifyType = NotifyType.INFO,
        body_format: NotifyFormat = NotifyFormat.TEXT,
        tag: str | Iterable[str] = "all",
        attach: str | None = None,
        interpret_escapes: bool | None = None,
        config: AppriseConfig | None = None,
        apprise_conn_id: str = AppriseHook.default_conn_name,
        **kwargs,
    ):
        if AIRFLOW_V_3_1_PLUS:
            #  Support for passing context was added in 3.1.0
            super().__init__(**kwargs)
        else:
            super().__init__()
        self.apprise_conn_id = apprise_conn_id
        self.body = body
        self.title = title
        self.notify_type = notify_type
        self.body_format = body_format
        self.tag = tag
        self.attach = attach
        self.interpret_escapes = interpret_escapes
        self.config = config

    @cached_property
    def hook(self) -> AppriseHook:
        """Apprise Hook."""
        return AppriseHook(apprise_conn_id=self.apprise_conn_id)

    def notify(self, context):
        """Send a alert to a apprise configured service."""
        self.hook.notify(
            body=self.body,
            title=self.title,
            notify_type=self.notify_type,
            body_format=self.body_format,
            tag=self.tag,
            attach=self.attach,
            interpret_escapes=self.interpret_escapes,
            config=self.config,
        )

    async def async_notify(self, context):
        """Send a alert to a apprise configured service."""
        await self.hook.async_notify(
            body=self.body,
            title=self.title,
            notify_type=self.notify_type,
            body_format=self.body_format,
            tag=self.tag,
            attach=self.attach,
            interpret_escapes=self.interpret_escapes,
            config=self.config,
        )


send_apprise_notification = AppriseNotifier



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

from __future__ import annotations

from unittest import mock

import pytest
from apprise import NotifyFormat, NotifyType

from airflow.models import Connection
from airflow.providers.apprise.notifications.apprise import (
    AppriseNotifier,
    send_apprise_notification,
)


class TestAppriseNotifier:
    @pytest.fixture(autouse=True)
    def setup_connections(self, create_connection_without_db):
        extra = {"config": {"path": "http://some_path_that_dont_exist/", "tag": "alert"}}
        create_connection_without_db(
            Connection(
                conn_id="apprise_default",
                conn_type="apprise",
                extra=extra,
            )
        )

    @mock.patch("airflow.providers.apprise.notifications.apprise.AppriseHook")
    def test_notifier(self, mock_apprise_hook, create_dag_without_db):
        notifier = send_apprise_notification(body="DISK at 99%", notify_type=NotifyType.FAILURE)
        notifier({"dag": create_dag_without_db("test_notifier")})
        call_args = mock_apprise_hook.return_value.notify.call_args.kwargs

        assert call_args == {
            "body": "DISK at 99%",
            "notify_type": NotifyType.FAILURE,
            "title": None,
            "body_format": NotifyFormat.TEXT,
            "tag": "all",
            "attach": None,
            "interpret_escapes": None,
            "config": None,
        }
        mock_apprise_hook.return_value.notify.assert_called_once()

    @mock.patch("airflow.providers.apprise.notifications.apprise.AppriseHook")
    def test_notifier_with_notifier_class(self, mock_apprise_hook, create_dag_without_db):
        notifier = AppriseNotifier(body="DISK at 99%", notify_type=NotifyType.FAILURE)
        notifier({"dag": create_dag_without_db("test_notifier")})
        call_args = mock_apprise_hook.return_value.notify.call_args.kwargs

        assert call_args == {
            "body": "DISK at 99%",
            "notify_type": NotifyType.FAILURE,
            "title": None,
            "body_format": NotifyFormat.TEXT,
            "tag": "all",
            "attach": None,
            "interpret_escapes": None,
            "config": None,
        }
        mock_apprise_hook.return_value.notify.assert_called_once()

    @mock.patch("airflow.providers.apprise.notifications.apprise.AppriseHook")
    def test_notifier_templated(self, mock_apprise_hook, create_dag_without_db):
        notifier = AppriseNotifier(
            notify_type=NotifyType.FAILURE,
            title="DISK at 99% {{dag.dag_id}}",
            body="System can crash soon {{dag.dag_id}}",
        )
        context = {"dag": create_dag_without_db("test_notifier")}
        notifier(context)
        call_args = mock_apprise_hook.return_value.notify.call_args.kwargs
        assert call_args == {
            "body": "System can crash soon test_notifier",
            "title": "DISK at 99% test_notifier",
            "notify_type": NotifyType.FAILURE,
            "body_format": NotifyFormat.TEXT,
            "tag": "all",
            "attach": None,
            "interpret_escapes": None,
            "config": None,
        }
        mock_apprise_hook.return_value.notify.assert_called_once()

    @pytest.mark.asyncio
    @mock.patch("airflow.providers.apprise.notifications.apprise.AppriseHook")
    async def test_async_apprise_notifier(self, mock_apprise_hook, create_dag_without_db):
        mock_apprise_hook.return_value.async_notify = mock.AsyncMock()

        notifier = send_apprise_notification(body="DISK at 99%", notify_type=NotifyType.FAILURE)

        await notifier.async_notify({"dag": create_dag_without_db("test_notifier")})

        call_args = mock_apprise_hook.return_value.async_notify.call_args.kwargs

        assert call_args == {
            "body": "DISK at 99%",
            "notify_type": NotifyType.FAILURE,
            "title": None,
            "body_format": NotifyFormat.TEXT,
            "tag": "all",
            "attach": None,
            "interpret_escapes": None,
            "config": None,
        }
        mock_apprise_hook.return_value.async_notify.assert_called_once()
